@@ -29,7 +29,7 @@ coordinates, like polar or spherical coordinates.
 """
 #pylint: disable=wildcard-import,no-name-in-module
 
-__version__ = '3.3.0'
+__version__ = '4.0.0'
 __author__ = 'Mikael Mortensen'
 
 import numpy as np
@@ -42,12 +42,14 @@ from . import laguerre
 from . import hermite
 from . import fourier
 from . import jacobi
+from . import ultraspherical
 from . import matrixbase
 from . import la
+from .coordinates import Coordinates
 from .fourier import energy_fourier
 from .io import *
 from .matrixbase import *
-from .spectralbase import inner_product, MixedFunctionSpace
+from .spectralbase import inner_product, MixedFunctionSpace, BoundaryConditions
 from .forms import *
 from .tensorproductspace import *
 from .utilities import *
