@@ -77,9 +77,8 @@ from __future__ import division
 import functools
 import numpy as np
 import sympy as sp
-from shenfun.optimization import cython, numba, optimizer
-from shenfun.matrixbase import SpectralMatrix, SparseMatrix, extract_diagonal_matrix
-from shenfun.la import SparseMatrixSolver
+from shenfun.optimization import cython, numba
+from shenfun.matrixbase import SpectralMatrix, extract_diagonal_matrix
 from shenfun.la import TDMA as generic_TDMA
 from shenfun.la import PDMA as generic_PDMA
 from shenfun.la import TwoDMA, FDMA
@@ -1160,7 +1159,7 @@ class BSBTmat(SpectralMatrix):
         k = np.arange(Q, dtype=float)
         d = {0: ck[:Q]*np.pi/2,
              2: -np.pi*(k[:min(Q, M-2)]+2)/(k[:min(Q, M-2)]+3),
-             4: 0.5*np.pi*ck[4:min(Q+4,M)]*(k[:min(Q, M-4)]+1)/(k[:min(Q, M-4)]+3)}
+             4: 0.5*np.pi*ck[4:min(Q+4, M)]*(k[:min(Q, M-4)]+1)/(k[:min(Q, M-4)]+3)}
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 class BCNCNmat(SpectralMatrix):
@@ -1219,16 +1218,16 @@ class BSDHHmat(SpectralMatrix):
         cp = np.ones(test[0].N); cp[2] = 2
         dk = np.ones(test[0].N); dk[:2] = 0
         d = {-2: -(np.pi/8)*ck[:-4],
-              0: (np.pi/8)*(ck[:-2]*(ck[:-2]+1)+dk[:-2]),
-              2: -(np.pi/8)*(ck[:-4]+2),
-              4: (np.pi/8)}
+             0: (np.pi/8)*(ck[:-2]*(ck[:-2]+1)+dk[:-2]),
+             2: -(np.pi/8)*(ck[:-4]+2),
+             4: (np.pi/8)}
 
         if trial[0].is_scaled():
             k = np.arange(test[0].N)
             d[-2] *= 1/(k[1:-3]*k[2:-2])
-            d[0]  *= 1/(k[1:-1]*k[2:])
-            d[2]  *= 1/((k[:-4]+3)*(k[:-4]+4))
-            d[4]  *= 1/((k[:-6]+5)*(k[:-6]+6))
+            d[0] *= 1/(k[1:-1]*k[2:])
+            d[2] *= 1/((k[:-4]+3)*(k[:-4]+4))
+            d[4] *= 1/((k[:-6]+5)*(k[:-6]+6))
 
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
@@ -1742,8 +1741,8 @@ class ASNSNmat(SpectralMatrix):
             s[0] = slice(None)
             j2 = j2[tuple(s)]
         vc = v[:-2]*j2
-        d0 = -2*np.pi*(k+1)/(k+2)
-        d2 = -4*np.pi*(k[:-2]+1)/(k[:-2]+2)**2
+        d0 = -2*np.pi*(k+1)/(k+2)*self.scale*self._keyscale
+        d2 = -4*np.pi*(k[:-2]+1)/(k[:-2]+2)**2*self.scale*self._keyscale
         c[N-1] = d0[N-1]*vc[N-1]
         c[N-2] = d0[N-2]*vc[N-2]
         s0 = 0
