@@ -1,11 +1,13 @@
 from numbers import Number, Integral
 from itertools import product
+from collections import defaultdict
 import copy
 from scipy.special import sph_harm, erf, airy
 import numpy as np
 import sympy as sp
 from shenfun.config import config
 from shenfun.optimization.cython import evaluate
+from shenfun.spectralbase import BoundaryConditions
 from mpi4py_fft import DistArray
 
 __all__ = ('Expr', 'BasisFunction', 'TestFunction', 'TrialFunction', 'Function',
@@ -38,42 +40,11 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
         - ``Jacobi`` or ``J``
 
     bc : tuple or dict, optional
-        Choose one of
-
-        * 2-tuple of numbers (a, b) - Dirichlet boundary condition with
-          :math:`v(-1)=a` and :math:`v(1)=b`.
-
-        * (None, a) - Dirichlet on right boundary, nothing on left.
-
-        * 4-tuple (a, b, c, d) - Biharmonic with the two non-zero Dirichlet
-          conditions first :math:`v(-1)=a` and :math:`v(1)=b` and then
-          the two Neumann.
-
-        * dict with keys 'left' and 'right', for left and right boundaries,
-          and a list of 2-tuples to specify the condition. This is the most
-          general form, and all boundary conditions may be specified like this.
-          Specify Dirichlet on both ends with
-
-              {'left': [('D', a)], 'right': [('D', b)]}
-
-          Specify mixed Neumann and Dirichlet as
-
-              {'left': [('N', a)], 'right': [('D', b)]}
-
-          For both conditions on the right do
-
-              {'right': [('N', a), ('D', b)]}
-
-          Note that not all combinations are possible for biharmonic
-          problems. One combination that is possible is a fixed free beam
-          with
-
-              {'left': [('D', a), ('N', b)], 'right': [('N2', c), ('N3', d)]}
-
-          where 'N2' and 'N3' represent second and third derivatives.
+        See :class:`~shenfun.spectralbase.BoundaryConditions`
 
     dtype : str or np.dtype, optional
         The datatype of physical space (input to forward transforms)
+
     quad : str, optional
         Type of quadrature
 
@@ -107,25 +78,29 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
     domain : two-tuple of floats, optional
         The computational domain
     scaled : bool
-        Whether to use scaled basis (only Legendre)
+        Whether to use scaled basis
     basis : str
         Name of basis to use, if there are more than one possible basis for a given
         boundary condition. For example, there are three Dirichlet bases for the
-        Chebyshev family: 'Heinricht', 'ShenDirichlet' and 'Phi1'
+        Chebyshev family: :class:`~.chebyshev.bases.Heinrichs`, :class:`~.chebyshev.bases.ShenDirichlet`
+        and :class:`~.chebyshev.bases.Phi1`
     padding_factor : float, optional
         For padding backward transform (for dealiasing)
     dealias_direct : bool, optional
         Use 2/3-rule dealiasing
     coordinates: 2- or 3-tuple (coordinate, position vector (, sympy assumptions)), optional
-        Map for curvilinear coordinatesystem.
-        The new coordinate variable in the new coordinate system is the first item.
-        Second item is a tuple for the Cartesian position vector as function of the
-        new variable in the first tuple. Example::
+        Map for curvilinear coordinates. Used for curves parametrized with one
+        parameter in :math:`\mathbb{R}^2` or :math:`\mathbb{R}^3`.
+        The parameter is the first item. Second item is a tuple of length 2 or
+        3 (for :math:`\mathbb{R}^2` or :math:`\mathbb{R}^3`) for the Cartesian
+        position vector. The sympy assumptions are there to help Sympy compute
+        basis vectors and metrics, see :class:`~shenfun.coordinates.Coordinates`
+        for options.
+        Example circle of radius 1::
 
-            theta = sp.Symbols('x', real=True, positive=True)
+            theta = sp.Symbol('x', real=True)
             rv = (sp.cos(theta), sp.sin(theta))
-
-        where theta and rv are the first and second items in the 2-tuple.
+            FunctionSpace(16, 'L', coordinates=(theta, rv), domain=(0, 2*np.pi))
 
     Examples
     --------
@@ -142,57 +117,6 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
     if domain is not None:
         par['domain'] = domain
 
-    def _process_bcs(bc, domain):
-
-        bcs = {'left': [], 'right': []}
-        df = 1
-        if domain is not None:
-            df = 2./(domain[1]-domain[0])
-        if isinstance(bc, tuple):
-            # Short form for Dirichlet or biharmonic with 2 or 4 numbers
-            assert len(bc) in (2, 4)
-            assert np.all([isinstance(i, (sp.Expr, Number)) or i is None for i in bc])
-            if len(bc) == 2:
-                if bc[0] is not None:
-                    bcs['left'] = [('D', bc[0])]
-                if bc[1] is not None:
-                    bcs['right'] = [('D', bc[1])]
-
-            if len(bc) == 4:
-                bcs['left'] = [('D', bc[0]), ('N', bc[1])]
-                bcs['right'] = [('D', bc[2]), ('N', bc[3])]
-            key = ['L'+bci[0] for bci in bcs['left']] + ['R'+bci[0] for bci in bcs['right']]
-
-        elif isinstance(bc, dict):
-            bcs = {k.lower(): list(v) if isinstance(v[0], (tuple, list)) else [v] for k, v in bc.items()}
-            bc = []
-            key = []
-            if 'left' in bcs:
-                bcs['left'].sort()
-                for bci in bcs['left']:
-                    if bci[0] == 'N':
-                        bc.append(bci[1]/df)
-                    elif bci[0] == 'N2':
-                        bc.append(bci[1]/df**2)
-                    elif bci[0] == 'N3':
-                        bc.append(bci[1]/df**3)
-                    else:
-                        bc.append(bci[1])
-                key += ['L'+bci[0] for bci in bcs['left']]
-            if 'right' in bcs:
-                bcs['right'].sort()
-                for bci in bcs['right']:
-                    if bci[0] == 'N':
-                        bc.append(bci[1]/df)
-                    elif bci[0] == 'N2':
-                        bc.append(bci[1]/df**2)
-                    elif bci[0] == 'N3':
-                        bc.append(bci[1]/df**3)
-                    else:
-                        bc.append(bci[1])
-                key += ['R'+bci[0] for bci in bcs['right']]
-        return key, tuple(bc)
-
     if family.lower() in ('fourier', 'f'):
         from shenfun import fourier
         if np.dtype(dtype).char in 'FDG':
@@ -215,53 +139,57 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
         # Boundary conditions abbreviated in dictionary keys as
         #   left->L, right->R, Dirichlet->D, Neumann->N
         # So LDRD means left Dirichlet, right Dirichlet
-        bases = {
-            '': chebyshev.bases.Orthogonal,
-            'LDRD': chebyshev.bases.ShenDirichlet,
-            'LNRN': chebyshev.bases.ShenNeumann,
-            'LDRN': chebyshev.bases.DirichletNeumann,
-            'LNRD': chebyshev.bases.NeumannDirichlet,
-            'RD': chebyshev.bases.UpperDirichlet,
-            'LD': chebyshev.bases.LowerDirichlet,
-            'RDRN': chebyshev.bases.UpperDirichletNeumann,
-            'LDLN': chebyshev.bases.LowerDirichletNeumann,
-            'LDLNRDRN': chebyshev.bases.ShenBiharmonic
-        }
-
-        if isinstance(bc, (tuple, dict)):
-            key, par['bc'] = _process_bcs(bc, domain)
-
-        elif bc is None:
-            key = ''
-
-        else:
-            raise NotImplementedError
+        bases = defaultdict(lambda: chebyshev.bases.Generic,
+                            {
+                                '': chebyshev.bases.Orthogonal,
+                                'LDRD': chebyshev.bases.ShenDirichlet,
+                                'LNRN': chebyshev.bases.ShenNeumann,
+                                'LDRN': chebyshev.bases.DirichletNeumann,
+                                'LNRD': chebyshev.bases.NeumannDirichlet,
+                                'RD': chebyshev.bases.UpperDirichlet,
+                                'LD': chebyshev.bases.LowerDirichlet,
+                                'RDRN': chebyshev.bases.UpperDirichletNeumann,
+                                'LDLN': chebyshev.bases.LowerDirichletNeumann,
+                                'LDLNRDRN': chebyshev.bases.ShenBiharmonic,
+                                'LDLNLN2RDRNRN2': chebyshev.bases.Phi3
+                            })
 
         if basis is not None:
             assert isinstance(basis, str)
             B = getattr(chebyshev.bases, basis)
+            if isinstance(bc, tuple) or 'Generic' in basis:
+                par['bc'] = bc
         else:
-            B = bases[''.join(key)]
+            if isinstance(bc, (str, tuple, dict)):
+                domain = (-1, 1) if domain is None else domain
+                bcs = BoundaryConditions(bc, domain=domain)
+                key = bcs.orderednames()
+                par['bc'] = bcs
+            elif bc is None:
+                key = ''
+            else:
+                raise RuntimeError
 
+            B = bases[''.join(key)]
         return B(N, **par)
 
     elif family.lower() in ('legendre', 'l'):
         from shenfun import legendre
 
-        bases = {
-            '': legendre.bases.Orthogonal,
-            'LDRD': legendre.bases.ShenDirichlet,
-            'LNRN': legendre.bases.ShenNeumann,
-            'LDRN': legendre.bases.DirichletNeumann,
-            'LNRD': legendre.bases.NeumannDirichlet,
-            'LD': legendre.bases.LowerDirichlet,
-            'RD': legendre.bases.UpperDirichlet,
-            'RDRN': legendre.bases.UpperDirichletNeumann,
-            'LNRDRN': legendre.bases.ShenBiPolar0,
-            'LDLNRDRN': legendre.bases.ShenBiharmonic,
-            'LDLNRN2RN3': legendre.bases.BeamFixedFree,
-            'LDLNRD': legendre.bases.DirichletNeumannDirichlet
-        }
+        bases = defaultdict(lambda: legendre.bases.Generic,
+                            {
+                                '': legendre.bases.Orthogonal,
+                                'LDRD': legendre.bases.ShenDirichlet,
+                                'LNRN': legendre.bases.ShenNeumann,
+                                'LDRN': legendre.bases.DirichletNeumann,
+                                'LNRD': legendre.bases.NeumannDirichlet,
+                                'LD': legendre.bases.LowerDirichlet,
+                                'RD': legendre.bases.UpperDirichlet,
+                                'RDRN': legendre.bases.UpperDirichletNeumann,
+                                'LDLNRDRN': legendre.bases.ShenBiharmonic,
+                                'LDLNLN2RDRNRN2': legendre.bases.Phi3,
+                                'LDLNRN2RN3': legendre.bases.BeamFixedFree,
+                            })
 
         if quad is not None:
             assert quad in ('LG', 'GL')
@@ -271,19 +199,21 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
             assert isinstance(scaled, bool)
             par['scaled'] = scaled
 
-        if isinstance(bc, (tuple, dict)):
-            key, par['bc'] = _process_bcs(bc, domain)
-
-        elif bc is None:
-            key = ''
-
-        else:
-            raise NotImplementedError
-
         if basis is not None:
             assert isinstance(basis, str)
             B = getattr(legendre.bases, basis)
+            if isinstance(bc, tuple):
+                par['bc'] = bc
         else:
+            if isinstance(bc, (str, tuple, dict)):
+                domain = (-1, 1) if domain is None else domain
+                bcs = BoundaryConditions(bc, domain=domain)
+                key = bcs.orderednames()
+                par['bc'] = bcs
+            elif bc is None:
+                key = ''
+            else:
+                raise RuntimeError
             B = bases[''.join(key)]
 
         return B(N, **par)
@@ -298,25 +228,72 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
             assert isinstance(scaled, bool)
             par['scaled'] = scaled
 
-        bases = {
-            '': chebyshevu.bases.Orthogonal,
-            'LDRD': chebyshevu.bases.CompactDirichlet,
-            'LDLNRDRN': chebyshevu.bases.Phi2
-        }
-
-        if isinstance(bc, (tuple, dict)):
-            key, par['bc'] = _process_bcs(bc, domain)
-
-        elif bc is None:
-            key = ''
-
-        else:
-            raise NotImplementedError
+        bases = defaultdict(lambda: chebyshevu.bases.Generic,
+                            {
+                                '': chebyshevu.bases.Orthogonal,
+                                'LDRD': chebyshevu.bases.CompactDirichlet,
+                                'LNRN': chebyshevu.bases.CompactNeumann,
+                                'LDLNRDRN': chebyshevu.bases.Phi2,
+                                'LDLNLN2RDRNRN2': chebyshevu.bases.Phi3,
+                                'LDLNLN2N3RDRNRN2N3': chebyshevu.bases.Phi4
+                            })
 
         if basis is not None:
             assert isinstance(basis, str)
             B = getattr(chebyshevu.bases, basis)
+            if isinstance(bc, tuple):
+                par['bc'] = bc
         else:
+            if isinstance(bc, (str, tuple, dict)):
+                domain = (-1, 1) if domain is None else domain
+                bcs = BoundaryConditions(bc, domain=domain)
+                key = bcs.orderednames()
+                par['bc'] = bcs
+
+            elif bc is None:
+                key = ''
+            else:
+                raise NotImplementedError
+            B = bases[''.join(key)]
+
+        return B(N, **par)
+
+    elif family.lower() in ('ultraspherical', 'q'):
+        from shenfun import ultraspherical
+        if quad is not None:
+            assert quad == 'QG'
+            par['quad'] = quad
+
+        if scaled is not None:
+            assert isinstance(scaled, bool)
+            par['scaled'] = scaled
+
+        bases = defaultdict(lambda: ultraspherical.bases.Generic,
+                            {
+                                '': ultraspherical.bases.Orthogonal,
+                                'LDRD': ultraspherical.bases.CompactDirichlet,
+                                'LNRN': ultraspherical.bases.CompactNeumann,
+                                'LDLNRDRN': ultraspherical.bases.Phi2,
+                                'LDLNLN2RDRNRN2': ultraspherical.bases.Phi3,
+                                'LDLNLN2N3RDRNRN2N3': ultraspherical.bases.Phi4
+                            })
+
+        if basis is not None:
+            assert isinstance(basis, str)
+            B = getattr(ultraspherical.bases, basis)
+            if isinstance(bc, tuple):
+                par['bc'] = bc
+        else:
+            if isinstance(bc, (str, tuple, dict)):
+                domain = (-1, 1) if domain is None else domain
+                bcs = BoundaryConditions(bc, domain=domain)
+                key = bcs.orderednames()
+                par['bc'] = bcs
+
+            elif bc is None:
+                key = ''
+            else:
+                raise NotImplementedError
             B = bases[''.join(key)]
 
         return B(N, **par)
@@ -324,23 +301,33 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
     elif family.lower() in ('laguerre', 'la'):
         from shenfun import laguerre
         if quad is not None:
-            assert quad in ('LG', 'GR')
+            assert quad in ('LG',)
             par['quad'] = quad
 
-        if bc is None:
-            B = laguerre.bases.Orthogonal
-
-        elif isinstance(bc, tuple):
-            assert len(bc) == 2
-            par['bc'] = bc
-            B = laguerre.bases.ShenDirichlet
-
-        elif isinstance(bc, str):
-            if bc.lower() == 'dirichlet':
-                B = laguerre.bases.ShenDirichlet
+        bases = defaultdict(lambda: laguerre.bases.Generic,
+                            {
+                                '': laguerre.bases.Orthogonal,
+                                'LD': laguerre.bases.CompactDirichlet,
+                                'LN': laguerre.bases.CompactNeumann,
+                            })
 
+        if basis is not None:
+            assert isinstance(basis, str)
+            B = getattr(laguerre.bases, basis)
+            if isinstance(bc, tuple):
+                par['bc'] = bc
         else:
-            raise NotImplementedError
+            if isinstance(bc, (str, tuple, dict)):
+                domain = (0, np.inf) if domain is None else domain
+                bcs = BoundaryConditions(bc, domain=domain)
+                key = bcs.orderednames()
+                par['bc'] = bcs
+
+            elif bc is None:
+                key = ''
+            else:
+                raise NotImplementedError
+            B = bases[''.join(key)]
 
         return B(N, **par)
 
@@ -374,22 +361,25 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
             assert isinstance(scaled, bool)
             par['scaled'] = scaled
 
-        bases = {
-            '': jacobi.bases.Orthogonal,
-            'LDRD': jacobi.bases.CompactDirichlet,
-            'LDLNRDRN': jacobi.bases.Phi2,
-            '6th order': jacobi.bases.ShenOrder6
-        }
+        bases = defaultdict(lambda: jacobi.bases.Generic,
+                            {
+                                '': jacobi.bases.Orthogonal,
+                                'LDRD': jacobi.bases.CompactDirichlet,
+                                'LNRN': jacobi.bases.CompactNeumann,
+                                'LDLNRDRN': jacobi.bases.Phi2,
+                                'LDLNLN2RDRNRN2': jacobi.bases.Phi3,
+                                'LDLNLN2N3RDRNRN2N3': jacobi.bases.Phi4,
+                            })
 
-        if isinstance(bc, (tuple, dict)):
-            key, par['bc'] = _process_bcs(bc, domain)
+        if isinstance(bc, (str, tuple, dict)):
+            domain = (-1, 1) if domain is None else domain
+            bcs = BoundaryConditions(bc, domain=domain)
+            key = bcs.orderednames()
+            par['bc'] = bcs
 
         elif bc is None:
             key = ''
 
-        elif bc == '6th order':
-            key = bc
-
         else:
             raise NotImplementedError
 
@@ -425,8 +415,8 @@ class Expr:
 
         - the outermost list represents a tensor component. There is one item for
           each tensor component. If the Expr is a scalar with rank = 0, then
-          len(terms) = 1. For vectors it equals the number of dimensions and
-          for second order tensors it equals ndim**2
+          len(terms) = 1. For vectors it equals ndim, which is the number of
+          dimensions, and for second order tensors it equals ndim*ndim.
 
         - the second nested list represents the different terms in the form, that
           may be more than one. For example, the scalar valued `div(grad(u))` has
@@ -539,6 +529,7 @@ class Expr:
             return 1
         if len(self._terms) == len(self._terms[0][0])**2:
             return 2
+        raise RuntimeError
 
     @property
     def tensor_rank(self):
@@ -664,9 +655,6 @@ class Expr:
         if basis is None:
             basis = 'u'
 
-        if self.expr_rank() > 0:
-            b = self.function_space().coors.get_covariant_basis()
-
         if isinstance(basis, str):
             # Create sympy Function
             st = basis
@@ -742,7 +730,7 @@ class Expr:
                 last_conj_index = -1
                 sl = -1
                 for axis, k in enumerate(b0):
-                    xx = test_sp[axis].map_reference_domain(np.squeeze(x[axis]))
+                    xx = np.atleast_1d(test_sp[axis].map_reference_domain(np.squeeze(x[axis])))
                     P = test_sp[axis].evaluate_basis_derivative_all(xx, k=k)
                     if not test_sp[axis].domain_factor() == 1:
                         P *= test_sp[axis].domain_factor()**(k)
@@ -794,7 +782,7 @@ class Expr:
         basis = self._basis
         if self.tensor_rank:
             if ((self.expr_rank() == self.tensor_rank and self.is_basis_function) or
-                (self.expr_rank() > self.tensor_rank and self.tensor_rank > 0)):
+                    (self.expr_rank() > self.tensor_rank and self.tensor_rank > 0)):
                 basis = self._basis[i]
 
         if self.expr_rank() == 1:
@@ -825,7 +813,7 @@ class Expr:
         Note
         ----
         The contravariant vector components are the components to the
-        covariant basisvectors. If the basis vectors are normalize, i.e.,
+        covariant basisvectors. If the basis vectors are normalized, i.e.,
         if the setting is config['basisvectors'] == 'normal', then the
         normal vector components must be scaled in order to get to the
         contravariant components.
@@ -845,7 +833,7 @@ class Expr:
         if config['basisvectors'] == 'normal':
             hi = self.function_space().coors.hi
             return self[i][j]*(1/(hi[i]*hi[j]))
-        assert config['basisvectors'] =='covariant'
+        assert config['basisvectors'] == 'covariant'
         return self[i][j]
 
     def __mul__(self, a):
@@ -1034,7 +1022,7 @@ class Expr:
 
 
 class BasisFunction:
-    """Base class for arguments to shenfun's Exprs
+    """Base class for arguments to shenfun's :class:`.Expr`
 
     Parameters
     ----------
@@ -1043,7 +1031,7 @@ class BasisFunction:
     index : int
         Local component of basis in :class:`.CompositeSpace`
     offset : int
-        The number of scalar spaces (i.e., :class:`.TensorProductSpace`es)
+        The number of scalar spaces (i.e., :class:`.TensorProductSpace`)
         ahead of this space
     base : The base :class:`BasisFunction`
     """
@@ -1093,7 +1081,7 @@ class BasisFunction:
     def offset(self):
         """Return offset of this basis
 
-        The offset is the number of scalar :class:`.TensorProductSpace`es ahead
+        The offset is the number of scalar :class:`.TensorProductSpace` es ahead
         of this space in a :class:`.CompositeSpace`.
         """
         return self._offset
@@ -1140,13 +1128,13 @@ class TestFunction(BasisFunction):
 
     Parameters
     ----------
-    space: :class:`TensorProductSpace` or :class:`CompositeSpace`
+    space: :class:`.TensorProductSpace` or :class:`.CompositeSpace`
     index: int, optional
         Component of basis in :class:`.CompositeSpace`
     offset : int
-        The number of scalar spaces (i.e., :class:`.TensorProductSpace`es)
+        The number of scalar spaces (i.e., :class:`.TensorProductSpace`)
         ahead of this space
-    base : The base :class:`TestFunction`
+    base : The base :class:`.TestFunction`
     """
 
     def __init__(self, space, index=0, offset=0, base=None):
@@ -1161,13 +1149,13 @@ class TrialFunction(BasisFunction):
 
     Parameters
     ----------
-    space: :class:`TensorProductSpace` or :class:`CompositeSpace`
+    space: :class:`.TensorProductSpace` or :class:`.CompositeSpace`
     index: int, optional
         Component of basis in :class:`.CompositeSpace`
     offset : int
-        The number of scalar spaces (i.e., :class:`.TensorProductSpace`es)
+        The number of scalar spaces (i.e., :class:`.TensorProductSpace`)
         ahead of this space
-    base : The base :class:`TrialFunction`
+    base : The base :class:`.TrialFunction`
     """
     def __init__(self, space, index=0, offset=0, base=None):
         BasisFunction.__init__(self, space, index, offset, base)
@@ -1395,7 +1383,7 @@ class ShenfunBaseArray(DistArray):
 
 class Function(ShenfunBaseArray, BasisFunction):
     r"""
-    Spectral Galerkin function for given :class:`.TensorProductSpace` or :func:`.SpectralBase`
+    Spectral Galerkin function for given :class:`.TensorProductSpace` or :class:`.SpectralBase`
 
     The Function in one dimension is
 
@@ -1432,7 +1420,7 @@ class Function(ShenfunBaseArray, BasisFunction):
 
     Parameters
     ----------
-    space : :class:`.TensorProductSpace` or :class:`.FunctionSpace`
+    space : :class:`.TensorProductSpace` or :func:`.FunctionSpace`
     val : int or float
         Value used to initialize array
     buffer : Numpy array, :class:`.Function` or sympy `Expr`
@@ -1446,7 +1434,9 @@ class Function(ShenfunBaseArray, BasisFunction):
         Absolute tolerance for adaptively finding dimension of space from
         the buffer
 
-    .. note:: For more information, see `numpy.ndarray <https://docs.scipy.org/doc/numpy/reference/generated/numpy.ndarray.html>`_
+    Note
+    ----
+    For more information, see `numpy.ndarray <https://docs.scipy.org/doc/numpy/reference/generated/numpy.ndarray.html>`_
 
     Example
     -------
@@ -1804,9 +1794,9 @@ class Function(ShenfunBaseArray, BasisFunction):
         This function does not copy the boundary dofs.
         """
         if dims is None:
-            dims = self.function_space().get_ndiag_cum_dofs()
+            dims = self.function_space()._get_ndiag_cum_dofs()
         if sl is None:
-            sl = self.function_space().get_ndiag_slices(j)
+            sl = self.function_space()._get_ndiag_slices(j)
         if f is None:
             f = np.zeros(dims[-1], dtype=self.dtype)
         for i, s in enumerate(sl):
@@ -1835,9 +1825,9 @@ class Function(ShenfunBaseArray, BasisFunction):
         This function does not copy the boundary dofs.
         """
         if dims is None:
-            dims = self.function_space().get_ndiag_cum_dofs()
+            dims = self.function_space()._get_ndiag_cum_dofs()
         if sl is None:
-            sl = self.function_space().get_ndiag_slices(j)
+            sl = self.function_space()._get_ndiag_slices(j)
         for i, s in enumerate(sl):
             si = tuple(s)
             self[si] = f[dims[i]:dims[i+1]].reshape(self[si].shape)
@@ -1904,7 +1894,9 @@ class Array(ShenfunBaseArray):
         A sympy expression is evaluated on the quadrature mesh and
         the result is used as buffer.
 
-    .. note:: For more information, see `numpy.ndarray <https://docs.scipy.org/doc/numpy/reference/generated/numpy.ndarray.html>`_
+    Note
+    ----
+    For more information, see `numpy.ndarray <https://docs.scipy.org/doc/numpy/reference/generated/numpy.ndarray.html>`_
 
     Examples
     --------
@@ -1930,7 +1922,7 @@ class Array(ShenfunBaseArray):
     def offset(self):
         """Return offset of this basis
 
-        The offset is the number of scalar :class:`.TensorProductSpace`es ahead
+        The offset is the number of scalar :class:`.TensorProductSpace` es ahead
         of this Arrays space in a :class:`.CompositeSpace`.
         """
         return self._offset
@@ -1946,7 +1938,6 @@ class Array(ShenfunBaseArray):
         assert self.tensor_rank == 1
         T = self.function_space()
         psi = T.coors.psi
-        ndim = len(psi)
         x = T.local_mesh(True)
         b = T.coors.get_basis()
         bij = np.array(sp.lambdify(psi, b)(*x), dtype=object)
@@ -1958,4 +1949,3 @@ class Array(ShenfunBaseArray):
         sl[1] = None
         uc = np.sum(self[tuple(sl)]*bij, axis=0)
         return uc
-
