@@ -10,54 +10,43 @@ from shenfun.spectralbase import SpectralBase, Transform, islicedict, slicedict
 
 
 class Orthogonal(SpectralBase):
-    r"""Base class for Hermite functions
+    r"""Function space for Hermite functions
+
+    The orthogonal basis is the Hermite function
+
+    .. math::
+
+        \phi_k = H_k \frac{1}{\pi^{0.25} \sqrt{2^n n!}} \exp(-x^2/2), \quad k = 0, 1, \ldots, N-1,
+
+    where :math:`\phi_k` and :math:`H_k` are the Hermite function and Hermite
+    polynomials of order k, respectively.
 
     Parameters
     ----------
-        N : int, optional
-            Number of quadrature points. Should be even for efficiency, but
-            this is not required.
-        quad : str, optional
-            Type of quadrature
-
-            - HG - Hermite-Gauss
-
-        domain : 2-tuple of floats, optional
-            The computational domain.
-        padding_factor : float, optional
-            Factor for padding backward transforms. padding_factor=1.5
-            corresponds to a 3/2-rule for dealiasing.
-        dealias_direct : bool, optional
-            True for dealiasing using 2/3-rule. Must be used with
-            padding_factor = 1.
-        dtype : data-type, optional
-            Type of input data in real physical space. Will be overloaded when
-            basis is part of a :class:`.TensorProductSpace`.
-        coordinates: 2- or 3-tuple (coordinate, position vector (, sympy assumptions)), optional
-            Map for curvilinear coordinatesystem.
-            The new coordinate variable in the new coordinate system is the first item.
-            Second item is a tuple for the Cartesian position vector as function of the
-            new variable in the first tuple. Example::
-
-                theta = sp.Symbols('x', real=True, positive=True)
-                rv = (sp.cos(theta), sp.sin(theta))
+    N : int, optional
+        Number of quadrature points. Should be even for efficiency, but
+        this is not required.
+    padding_factor : float, optional
+        Factor for padding backward transforms. padding_factor=1.5
+        corresponds to a 3/2-rule for dealiasing.
+    dealias_direct : bool, optional
+        True for dealiasing using 2/3-rule. Must be used with
+        padding_factor = 1.
+    dtype : data-type, optional
+        Type of input data in real physical space. Will be overloaded when
+        basis is part of a :class:`.TensorProductSpace`.
+    coordinates: 2- or 3-tuple (coordinate, position vector (, sympy assumptions)), optional
+        Map for curvilinear coordinatesystem, and parameters to :class:`~shenfun.coordinates.Coordinates`
 
     Note
     ----
     We are using Hermite functions and not the regular Hermite polynomials
-    as basis functions. A Hermite function is defined as
-
-    .. math::
-
-        H_k = P_k \cdot \frac{1}{\pi^{0.25} \sqrt{2^n n!}} \exp(-x^2/2)
-
-    where :math:`H_k` and :math:`P_k` are the Hermite function and Hermite
-    polynomials of order k, respectively.
+    as basis functions.
 
     """
-    def __init__(self, N, quad="HG", bc=(0., 0.), dtype=float,
-                 padding_factor=1, dealias_direct=False, coordinates=None, **kw):
-        SpectralBase.__init__(self, N, quad=quad, domain=(-np.inf, np.inf), dtype=dtype,
+    def __init__(self, N, dtype=float, padding_factor=1, dealias_direct=False,
+                 coordinates=None, **kw):
+        SpectralBase.__init__(self, N, quad="HG", domain=(-np.inf, np.inf), dtype=dtype,
                               padding_factor=padding_factor, dealias_direct=dealias_direct,
                               coordinates=coordinates)
         self.forward = functools.partial(self.forward, fast_transform=False)
