@@ -3,14 +3,12 @@ This module contains classes for working with sparse matrices.
 
 """
 from __future__ import division
-from typing import List
 from copy import copy, deepcopy
 from collections.abc import Mapping, MutableMapping
-from numbers import Number, Integral
+from numbers import Number
 import numpy as np
 import sympy as sp
 from scipy.sparse import bmat, dia_matrix, csr_matrix, kron, diags as sp_diags
-from scipy.sparse.linalg import spsolve
 from mpi4py import MPI
 from shenfun.config import config
 from .utilities import integrate_sympy
@@ -48,7 +46,7 @@ class SparseMatrix(MutableMapping):
     using the `diags` method. However, Scipy's matrices are not implemented to
     act along different axes of multidimensional arrays, which is required
     for tensor product matrices, see :class:`.TPMatrix`. Hence the need for
-    this SpectralMatrix class.
+    this SparseMatrix class.
 
     Examples
     --------
@@ -106,7 +104,7 @@ class SparseMatrix(MutableMapping):
              - cython - Cython implementation that may be implemented in subclass
              - numba - Numba implementation that may be implemented in subclass
 
-             Using config['matrix']['sparse']['matvec'] setting if format is None
+             Using ``config['matrix']['sparse']['matvec']`` setting if format is None
 
         axis : int, optional
             The axis over which to take the matrix vector product
@@ -156,7 +154,7 @@ class SparseMatrix(MutableMapping):
             - csr - Compressed sparse row
             - csc - Compressed sparse column
 
-            Using config['matrix']['sparse']['diags'] setting if format is None
+            Using ``config['matrix']['sparse']['diags']`` setting if format is None
 
         scaled : bool, optional
             Return matrix scaled by the constant self.scale if True
@@ -395,6 +393,7 @@ class SparseMatrix(MutableMapping):
             c *= sc
 
     def incorporate_scale(self):
+        """Modifies matrix such that self.scale = 1"""
         if abs(self.scale-1) < 1e-8:
             return
         if hasattr(self, '_keyscale'):
@@ -468,14 +467,14 @@ class SparseMatrix(MutableMapping):
 
     def isdiagonal(self):
         if len(self) == 1:
-            if (0 in self):
+            if 0 in self:
                 return True
         return False
 
     def isidentity(self):
         if not len(self) == 1:
             return False
-        if (0 not in self):
+        if 0 not in self:
             return False
         d = self[0]
         if np.all(d == 1):
@@ -491,7 +490,7 @@ class SparseMatrix(MutableMapping):
         for key, val in self.items():
             if key <= 0:
                 continue
-            if not np.all(abs(self[key]-self[-key]) < 1e-16):
+            if not np.all(abs(val-self[-key]) < 1e-16):
                 return False
         return True
 
@@ -533,6 +532,7 @@ class SpectralMatrix(SparseMatrix):
         - :mod:`.legendre.bases`
         - :mod:`.chebyshev.bases`
         - :mod:`.chebyshevu.bases`
+        - :mod:`.ultraspherical.bases`
         - :mod:`.fourier.bases`
         - :mod:`.laguerre.bases`
         - :mod:`.hermite.bases`
@@ -673,7 +673,7 @@ class Identity(SparseMatrix):
         SparseMatrix.__init__(self, {0:1}, shape, scale)
         self.measure = 1
 
-    def solve(self, b, u=None, axis=0):
+    def solve(self, b, u=None, axis=0, constraints=()):
         if u is None:
             u = b
         else:
@@ -708,7 +708,7 @@ class ScipyMatrix(csr_matrix):
             The axis over which to take the matrix vector product
 
         """
-        N, M = self.shape
+        M = self.shape[1]
         c.fill(0)
 
         # Roll relevant axis to first
@@ -878,7 +878,13 @@ class BlockMatrix:
                         self.mats[i][j].append(m)
 
     def get_mats(self, return_first=False):
-        """Return flattened list of matrices in self"""
+        """Return flattened list of matrices in self
+
+        Parameters
+        ----------
+        return_first : bool, optional
+            Return just the first matrix in the loop if True
+        """
         tpmats = []
         for mi in self.mats:
             for mij in mi:
@@ -894,7 +900,12 @@ class BlockMatrix:
     def matvec(self, v, c, format=None, use_scipy=None):
         """Compute matrix vector product
 
-            c = self * v
+        .. math::
+
+            c = A v
+
+        where :math:`A` is the self block matrix and :math:`v,c` are flattened
+        instances of the class :class:`.Function`.
 
         Parameters
         ----------
@@ -907,7 +918,7 @@ class BlockMatrix:
         use_scipy : boolean, optional
             Whether to assemble and use scipy's bmat for the matvec, or to use
             the matvec methods of this BlockMatrix's TPMatrices.
-            Using config['matrix']['block']['use_scipy'] if use_scipy is None
+            Using ``config['matrix']['block']['use_scipy']`` if use_scipy is None
         Returns
         -------
         c : :class:`.Function`
@@ -928,8 +939,8 @@ class BlockMatrix:
             daxes = self.testbase.get_diagonal_axes()
             if len(daxes) > 0:
                 daxes += 1
-            sl1, dims1 = self.trialbase.get_ndiag_slices_and_dims()
-            sl2, dims2 = self.testbase.get_ndiag_slices_and_dims()
+            sl1, dims1 = self.trialbase._get_ndiag_slices_and_dims()
+            sl2, dims2 = self.testbase._get_ndiag_slices_and_dims()
             gi = np.zeros(dims1[-1], dtype=v.dtype)
             go = np.zeros(dims2[-1], dtype=v.dtype)
             for key, val in self._Ai.items():
@@ -961,7 +972,6 @@ class BlockMatrix:
         return self.mats[ij[0]][ij[1]]
 
     def contains_bc_matrix(self):
-        """Return True if self contains a boundary TPMatrix"""
         for mi in self.mats:
             for mij in mi:
                 if isinstance(mij, (list, tuple)):
@@ -986,12 +996,11 @@ class BlockMatrix:
         ----------
         format : str or None, optional
             The format of the sparse scipy matrix. Using
-            config['matrix']['block']['assemble'] if None.
+            ``config['matrix']['block']['assemble']`` if None.
         """
         if self._Ai is not None:
             return
         self._Ai = {}
-        tpmat = self.get_mats(True)
         N = self.testbase.forward.output_array.shape
         daxes = self.get_diagonal_axes()
         ndindices = [(0,)] if len(daxes) == 0 else np.ndindex(tuple(np.array(N)[daxes]))
@@ -1023,7 +1032,7 @@ class BlockMatrix:
             axes, or the axes with Fourier bases.
         format : str or None, optional
             The format of the returned matrix. See `Scipy sparse matrices <https://docs.scipy.org/doc/scipy/reference/sparse.html>`_
-            If None, then use default for TPMatrix.
+            If None, then use default for :class:`.TPMatrix`.
 
         """
         from .spectralbase import MixedFunctionSpace
@@ -1168,8 +1177,6 @@ class TPMatrix:
         self._issimplified = False
 
     def get_simplified(self):
-        """Return a version of self simplified by putting diagonal matrices in a
-        scale array"""
         diagonal_axes = np.setxor1d(self.naxes, range(self.space.dimensions)).astype(int)
         if len(diagonal_axes) == 0 or self._issimplified:
             return self
@@ -1318,13 +1325,6 @@ class TPMatrix:
         return c
 
     def get_key(self):
-        """Return key of the one nondiagonal matrix in the TPMatrix
-
-        Note
-        ----
-        Raises an error of there are more than one single nondiagonal matrix
-        in TPMatrix.
-        """
         naxis = self.space.get_nondiagonal_axes()
         assert len(naxis) == 1
         return self.mats[naxis[0]].get_key()
@@ -1352,10 +1352,10 @@ class TPMatrix:
             return TPMatrix(self.mats, self.space, self.trialspace, self.scale*a,
                             self.global_index, self.testbase, self.trialbase)
 
-        elif isinstance(a, np.ndarray):
-            c = np.empty_like(a)
-            c = self.matvec(a, c)
-            return c
+        assert isinstance(a, np.ndarray)
+        c = np.empty_like(a)
+        c = self.matvec(a, c)
+        return c
 
     def __rmul__(self, a):
         """Returns copy of self.__rmul__(a) <==> a*self"""
@@ -1535,6 +1535,7 @@ def check_sanity(A, test, trial, measure=1):
         - :mod:`.legendre.bases`
         - :mod:`.chebyshev.bases`
         - :mod:`.chebyshevu.bases`
+        - :mod:`.ultraspherical.bases`
         - :mod:`.fourier.bases`
         - :mod:`.laguerre.bases`
         - :mod:`.hermite.bases`
@@ -1566,6 +1567,7 @@ def get_dense_matrix(test, trial, measure=1):
         - :mod:`.legendre.bases`
         - :mod:`.chebyshev.bases`
         - :mod:`.chebyshevu.bases`
+        - :mod:`.ultraspherical.bases`
         - :mod:`.fourier.bases`
         - :mod:`.laguerre.bases`
         - :mod:`.hermite.bases`
@@ -1625,6 +1627,7 @@ def get_denser_matrix(test, trial, measure=1):
         - :mod:`.legendre.bases`
         - :mod:`.chebyshev.bases`
         - :mod:`.chebyshevu.bases`
+        - :mod:`.ultraspherical.bases`
         - :mod:`.fourier.bases`
         - :mod:`.laguerre.bases`
         - :mod:`.hermite.bases`
@@ -1740,6 +1743,7 @@ def get_dense_matrix_sympy(test, trial, measure=1):
         - :mod:`.legendre.bases`
         - :mod:`.chebyshev.bases`
         - :mod:`.chebyshevu.bases`
+        - :mod:`.ultraspherical.bases`
         - :mod:`.fourier.bases`
         - :mod:`.laguerre.bases`
         - :mod:`.hermite.bases`
@@ -1820,6 +1824,7 @@ def get_dense_matrix_quadpy(test, trial, measure=1):
         - :mod:`.legendre.bases`
         - :mod:`.chebyshev.bases`
         - :mod:`.chebyshevu.bases`
+        - :mod:`.ultraspherical.bases`
         - :mod:`.fourier.bases`
         - :mod:`.laguerre.bases`
         - :mod:`.hermite.bases`
