@@ -13,10 +13,11 @@ import sympy as sp
 from scipy.fftpack import dct
 from shenfun.optimization import optimizer
 from shenfun.config import config
+from .findbasis import get_bc_basis, get_stencil_matrix
 
 __all__ = ['dx', 'clenshaw_curtis1D', 'CachedArrayDict', 'surf3D', 'wrap_periodic',
            'outer', 'dot', 'apply_mask', 'integrate_sympy', 'mayavi_show',
-           'quiver3D']
+           'quiver3D', 'get_bc_basis', 'get_stencil_matrix']
 
 
 def dx(u, weighted=False):
@@ -29,8 +30,8 @@ def dx(u, weighted=False):
     Parameters
     ----------
 
-        u : Array
-            The Array to integrate
+    u : Array
+        The Array to integrate
 
     Note
     ----
@@ -287,7 +288,6 @@ def dot(u, v, output_array=None, forward_output=True):
             raise NotImplementedError
 
     else:
-        from shenfun import config
         gij = Vu.coors.get_metric_tensor(config['basisvectors'])
         mesh = Vup.local_mesh(True)
         def measure(g):
@@ -306,7 +306,7 @@ def dot(u, v, output_array=None, forward_output=True):
                 for i in range(D):
                     ui = ua[i]
                     for j in range(D):
-                        g = g[i, j]
+                        g = gij[i, j]
                         if not g == 0:
                             uv += ui*va[j]*measure(g)
 
@@ -458,6 +458,8 @@ def mayavi_show():
 def wrap_periodic(xs, axes=()):
     """Return arrays wrapped around periodically
 
+    Parameters
+    ----------
     xs : array or sequence of arrays
     axes : sequence of integers, optional
         Extend arrays in xs by one in direction given by wrap
@@ -533,7 +535,7 @@ def surf3D(u, mesh=None, backend='plotly', wrapaxes=(), slices=None, fig=None, k
 
     return fig
 
-def quiver3D(u, mesh=None, wrapaxes=(), new_figure=False, slices=None, fig=None, kind='normal', **kw):
+def quiver3D(u, mesh=None, wrapaxes=(), slices=None, fig=None, kind='normal', **kw):
     """
 
     Parameters
@@ -580,6 +582,11 @@ def quiver3D(u, mesh=None, wrapaxes=(), new_figure=False, slices=None, fig=None,
 
     x, y, z = wrap_periodic([x, y, z], wrapaxes)
     u = wrap_periodic([u], np.array(wrapaxes)+1)
+    if slices:
+        x = x[slices]
+        y = y[slices]
+        z = z[slices]
+        u = u[slices]
     if fig is None:
         mlab.figure(bgcolor=(1, 1, 1), size=(400, 400))
     mlab.quiver3d(x, y, z, u[0], u[1], u[2], **par)
