@@ -28,8 +28,8 @@ as they assume all matrices are diagonal.
 import types
 import numpy as np
 from shenfun import Function, TPMatrix, TrialFunction, TestFunction,\
-    inner, la, Expr, CompositeSpace, BlockMatrix, extract_bc_matrices,\
-    SparseMatrix, get_simplified_tpmatrices, ScipyMatrix
+    inner, la, Expr, CompositeSpace, BlockMatrix, SparseMatrix, \
+    get_simplified_tpmatrices, ScipyMatrix
 
 __all__ = ('IRK3', 'BackwardEuler', 'RK4', 'ETDRK4', 'ETD')
 
@@ -165,7 +165,7 @@ class IRK3(IntegratorBase):
                 else:
                     raise NotImplementedError
             else:
-                self.solver.append(BlockMatrixSolver(mats))
+                self.solver.append(la.BlockMatrixSolver(mats))
 
             if u0:
                 rhs_mats = inner(v, u+((a[rk]+b[rk])*dt/2)*u0)
@@ -260,7 +260,7 @@ class BackwardEuler(IntegratorBase):
             else:
                 raise NotImplementedError
         else:
-            self.solver = BlockMatrixSolver(mats)
+            self.solver = la.BlockMatrixSolver(mats)
         self.rhs_mats = BlockMatrix(M if isinstance(M, list) else [M])
 
     def compute_rhs(self, u, u_hat, dU, dU1):
@@ -420,7 +420,8 @@ class ETDRK4(IntegratorBase):
         L = self.LinearRHS(u, **self.params)
         if isinstance(L, Expr):
             L = inner(v, L)
-            L = get_simplified_tpmatrices(L)[0]
+            if isinstance(L, list):
+                L = get_simplified_tpmatrices(L)[0]
         if isinstance(L, list):
             assert self.T.tensor_rank == 1
             assert L[0].isidentity()
